#!/usr/bin/env python3
"""
Module GROUPING -- Simple Blog Grouping Extension
Sub-Package SIMPLEBLOG.EXTENSIONS
Copyright (C) 2012-2013 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information
"""

from itertools import groupby
from operator import attrgetter

from simpleblog import extendable_property
from simpleblog.extensions import BlogExtension, PageMixin


class GroupingPageMixin(PageMixin):
    
    config_vars = dict(
        groupindex_key='groupindex',
        group_key='datestamp_formatted',
        group_formats=dict(
            vartype=set,
            default=["html"])
    )
    
    @extendable_property()
    def group_head_template(self):
        return self.template_data("group", "head")
    
    @extendable_property()
    def group_foot_template(self):
        return self.template_data("group", "foot")
    
    def _get_format_entries(self):
        if self.format in self.group_formats:
            for groupindex, (groupkey, group) in enumerate(groupby(
                self.entries, attrgetter(self.group_key)
            )):
                params = {
                    self.groupindex_key: groupindex,
                    self.group_key: groupkey
                }
                yield self.group_head_template.format(**params)
                group_params = dict(
                    groupindex=groupindex,
                    groupkey=groupkey
                )
                for entry in group:
                    entryparams = self.entry_params(entry)
                    entryparams.update(group_params)
                    yield entry.formatted(self.format, entryparams)
                yield self.group_foot_template.format(**params)
        else:
            for item in super(GroupingPageMixin, self)._get_format_entries():
                yield item


class GroupingExtension(BlogExtension):
    """Group page entries by date.
    """
    pass



#!/usr/bin/env python3
"""
Module ARCHIVES -- Simple Blog Archives Extension
Sub-Package SIMPLEBLOG.EXTENSIONS
Copyright (C) 2012-2013 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information
"""

from collections import defaultdict
from operator import attrgetter

from plib.stdlib.localize import monthname, monthname_long

from simpleblog import BlogEntries
from simpleblog.extensions import BlogExtension


class BlogArchiveEntries(BlogEntries):
    """Archive of entries spanning a given time period.
    """
    
    config_vars = dict(
        prefix=('archives_prefix', ""),
        archive_use_monthnames=False,
        archive_long_monthnames=False,
        archive_year_template="{year}",
        archive_month_template="{year}-{monthkey}",
        archive_day_template="{year}-{monthkey}-{daykey}"
    )
    
    sourcetype = 'archive'
    
    title_varnames = (
        'year',
        'month',
        'day',
        'monthkey',
        'daykey'
    )
    
    heading_varnames = (
        'title',
    )
    
    default_heading = "Archive: {title}"
    
    def __init__(self, blog, year, month=0, day=0):
        BlogEntries.__init__(self, blog)
        self.year = year
        self.month = month
        if month:
            self.monthname = mname = monthname(month)
            self.monthname_long = mname_long = monthname_long(month)
            self.monthkey = monthkey = (
                "{:02d}".format(month) if not self.archive_use_monthnames else
                mname if not self.archive_long_monthnames else
                mname_long
            )
        else:
            self.monthname = self.monthname_long = self.monthkey = ""
        self.day = day
        self.daykey = daykey = "{:02d}".format(day)
        
        if month:
            if day:
                self.sortkey = (year, month, day)
                self.urlshort = "/{0}/{1}/{2}/".format(year, monthkey, daykey)
                self.default_title = self.archive_day_template
            else:
                self.sortkey = (year, month)
                self.urlshort = "/{0}/{1}/".format(year, monthkey)
                self.default_title = self.archive_month_template
        else:
            self.sortkey = (year,)
            self.urlshort = "/{}/".format(year)
            self.default_title = self.archive_year_template
        
        if self.prefix:
            self.urlshort = "/{0}{1}".format(self.prefix, self.urlshort)
    
    def _get_entries(self):
        if self.month:
            if self.day:
                return self.blog.day_entries[(self.year, self.month, self.day)]
            return self.blog.month_entries[(self.year, self.month)]
        return self.blog.year_entries[self.year]
    
    def _get_sourcetype_sources(self):
        archives = self.blog.all_archives
        if self.month:
            if self.day:
                match = lambda a: (a.year == self.year) and (a.month == self.month)
            else:
                match = lambda a: (a.year == self.year) and (a.day == 0)
        else:
            match = lambda a: (a.month == 0) and (a.day == 0)
        return sorted((a for a in archives if match(a)), key=attrgetter('sortkey'))


class ArchivesExtension(BlogExtension):
    """Add blog archive pages.
    """
    
    config_vars = dict(
        archive_years=False,
        archive_months=False,
        archive_days=False,
        archive_link_years=False,
        archive_link_months=False,
        archive_link_days=False
    )
    
    def blog_mod_sources(self, blog, sources):
        
        blog.year_entries = defaultdict(list)
        blog.month_entries = defaultdict(list)
        blog.day_entries = defaultdict(list)
        
        def archive_containers(t):
            if self.archive_years:
                yield blog.year_entries[t.year]
            if self.archive_months:
                yield blog.month_entries[(t.year, t.month)]
            if self.archive_days:
                yield blog.day_entries[(t.year, t.month, t.day)]
        
        for entry in blog.all_entries:
            for container in archive_containers(entry.timestamp):
                container.append(entry)
        
        blog.all_archives = []
        archive_links = []
        
        if self.archive_years:
            years = [
                BlogArchiveEntries(blog, year)
                for year in blog.year_entries
            ]
            blog.all_archives.extend(years)
            if self.archive_link_years:
                archive_links.extend(years)
        
        if self.archive_months:
            months = [
                BlogArchiveEntries(blog, year, month)
                for year, month in blog.month_entries
            ]
            blog.all_archives.extend(months)
            if self.archive_link_months:
                archive_links.extend(months)
        
        if self.archive_days:
            days = [
                BlogArchiveEntries(blog, year, month, day)
                for year, month, day in blog.day_entries
            ]
            blog.all_archives.extend(days)
            if self.archive_link_days:
                archive_links.extend(days)
        
        blog.metadata.update(
            archive_links=self.get_links(archive_links, True)
        )
        
        sources.extend(
            (archive, "html")
            for archive in blog.all_archives
        )
        
        return sources



#!/usr/bin/env python3
"""
Module TIMEZONE -- Simple Blog Time Zone Extension
Sub-Package SIMPLEBLOG.EXTENSIONS
Copyright (C) 2012-2013 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information
"""

from datetime import datetime, timedelta

import pytz

from plib.stdlib.tztools import local_tzname

from simpleblog import extendable_property
from simpleblog.extensions import BlogExtension, EntryMixin


class TimezoneEntryMixin(EntryMixin):
    
    config_vars = dict(
        utc_timestamps=False,
        timezone_name=""
    )
    
    _tzname = ""
    
    @extendable_property()
    def timezone_tzname(self):
        return self._tzname or self.timezone_name or ("UTC" if self.utc_timestamps else local_tzname())
    
    @extendable_property()
    def timezone(self):
        return pytz.timezone(self.timezone_tzname)


class TimezoneExtension(BlogExtension):
    """Assign a specific time zone to your blog for displaying timestamps.
    
    This extension allows you to specify a timezone name for your blog,
    from among the allowed names in the ``pytz`` library, which is required
    if you want to use this extension.
    
    Note that this extension ignores the ``utc_timestamps`` config setting
    if the ``timezone_name`` config setting is present; if it is absent,
    and ``utc_timestamps`` is true, the pytz UTC timezone will be used;
    otherwise, the pytz timezone whose name corresponds with the name of
    the system's local time zone is used, if it can be found (if not, an
    exception is raised).
    """
    
    config_vars = dict(
        warn_on_timezone_mismatch=False,
    )
    
    def entry_get_datetime_from_mtime(self, entry, mtime):
        dt_naive = datetime.utcfromtimestamp(entry.mtime)
        # Can't use datetime constructor with pytz tzinfo object, per pytz docs
        # (UTC is supposed to work OK, but we'll take no chances), so we build
        # a new "naive" UTC datetime and localize it using the pytz API
        dt_utc = pytz.utc.localize(dt_naive)
        return dt_utc.astimezone(entry.timezone)
    
    def entry_get_string_from_datetime(self, entry, dt, fmt):
        assert dt.dst() is not None  # make sure dt is not "naive"
        idst = 0 if dt.dst() == timedelta(0) else 1
        return "{} {} {}".format(dt.strftime(fmt), entry.timezone_tzname, idst)
    
    def entry_get_datetime_from_string(self, entry, s, fmt):
        s, tzname, sdst = s.rsplit(' ', 2)
        if tzname != entry.timezone_tzname:
            # This flags the entry as having a "mismatched" time zone (i.e., a
            # different time zone from the blog's current default); this is why
            # we only do this if there is indeed a mismatch
            if self.warn_on_timezone_mismatch:
                # FIXME: Allow control of debug output/logging
                print("Mismatched time zone in entry!")
            entry._tzname = tzname
        dt_naive = datetime.strptime(s, fmt)
        return entry.timezone.localize(dt_naive, is_dst=bool(sdst))



#!/usr/bin/env python3
"""
Module PAGINATE -- Simple Blog Pagination Extension
Sub-Package SIMPLEBLOG.EXTENSIONS
Copyright (C) 2012-2013 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information
"""

from plib.stdlib.decotools import cached_method

from simpleblog import BlogEntries, noresult, newline
from simpleblog.extensions import BlogExtension


def num_pages(source, max_entries):
    """Return number of pages required to paginate source.
    """
    q, r = divmod(len(source.entries), max_entries)
    return q + int(r > 0)


class PageEntries(BlogEntries):
    """Entries for a particular page of a paginated source.
    """
    
    config_vars = dict(
        page_max_entries=10,
        page_home_include_pagenum=False,
        page_title_template="{title} - Page {pagenum}",
        page_heading_template="{heading} - Page {pagenum}",
        page_newer_label="Newer Entries",
        page_older_label="Older Entries",
    )
    
    def __init__(self, blog, source, pagenum):
        BlogEntries.__init__(self, blog)
        self.orig_source = source
        self.pagenum = pagenum
        self.index_start = pagenum * self.page_max_entries
        self.index_end = self.index_start + self.page_max_entries
        self.urlshort = source.urlshort
        
        if (pagenum == 0) and not self.page_home_include_pagenum:
            self.default_title = source.title
            self.default_heading = source.heading
        else:
            page_attrs = dict(
                title=source.title,
                heading=source.heading,
                pagenum=pagenum + 1  # pagenum is zero-based
            )
            self.default_title = self.page_title_template.format(**page_attrs)
            self.default_heading = self.page_heading_template.format(**page_attrs)
    
    def _get_entries(self):
        return self.orig_source.entries[self.index_start:self.index_end]
    
    @cached_method
    def make_urlpath(self, urlshort, pagenum):
        if (pagenum != self.pagenum) and (pagenum == 0):
            return urlshort
        return "{}index{}".format(
            urlshort,
            pagenum if pagenum > 0 else ""
        )
    
    def _get_urlpath(self):
        if self.urlshort is not None:
            return self.make_urlpath(
                self.urlshort,
                self.pagenum
            )
        raise NotImplementedError
    
    @cached_method
    def make_pagelinks(self, format):
        linkspecs = (
            (0, -1, self.page_newer_label),
            (num_pages(self.orig_source, self.page_max_entries) - 1, 1, self.page_older_label)
        )
        return tuple(
            '<a href="{}{}">{}</a>'.format(
                self.make_urlpath(self.urlshort, self.pagenum + ofs),
                ".{}".format(format) if (self.pagenum + ofs) > 0 else "",
                label
            ) if (self.pagenum * ofs) < test else ""
            for test, ofs, label in linkspecs
        )


class PaginateExtension(BlogExtension):
    """Paginate all blog sources with multiple entries.
    """
    
    config_vars = dict(
        paginate_formats=dict(
            vartype=set,
            default=["html"]),
        page_max_entries=10,
        page_force_short=True,
        page_links_include_sources=False,
        page_link_sep="&nbsp;&nbsp;"
    )
    
    def page_get_link_source(self, page):
        if isinstance(page.source, PageEntries):
            return page.source.orig_source
        return noresult
    
    def page_mod_entry_params(self, page, params, entry):
        if self.page_force_short:
            try:
                pagenum = page.source.pagenum
            except AttributeError:
                pass
            else:
                if pagenum > 0:
                    params.update(
                        force_short=True
                    )
        return params
    
    def page_mod_attrs(self, page, attrs):
        if isinstance(page.source, PageEntries):
            page_link_newer, page_link_older = page.source.make_pagelinks(page.format)
        else:
            page_link_newer = page_link_older = ""
        if self.page_links_include_sources:
            links = (
                attrs['page_sourcelink_next'],
                page_link_newer,
                page_link_older,
                attrs['page_sourcelink_prev']
            )
        else:
            links = (page_link_newer, page_link_older)
        page_links = self.page_link_sep.join(link for link in links if link)
        attrs.update(
            page_links="{}{}".format(page_links, newline) if page_links else "",
            page_link_newer=page_link_newer,
            page_link_older=page_link_older
        )
        return attrs
    
    @cached_method
    def paginate(self, source, format):
        return (
            format in self.paginate_formats
            and isinstance(source, BlogEntries)
            and (len(source.entries) > self.page_max_entries)
        )
    
    def blog_mod_sources(self, blog, sources):
        newsources = []
        for source, format in sources:
            if self.paginate(source, format):
                newsources.extend(
                    (PageEntries(blog, source, pagenum), format)
                    for pagenum in range(num_pages(source, self.page_max_entries))
                )
            else:
                newsources.append((source, format))
        return newsources



#!/usr/bin/env python3
"""
Sub-Package SIMPLEBLOG.EXTENSIONS
Copyright (C) 2012-2013 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information
"""

import sys
from operator import attrgetter

from plib.stdlib.builtins import first
from plib.stdlib.classtools import first_subclass
from plib.stdlib.decotools import cached_property, wraps_class
from plib.stdlib.iters import prefixed_items

from simpleblog import (
    BlogConfigUserMeta, BlogConfigUser,
    extension_types, extension_map, extend_attributes,
    BlogEntries, newline
)


class NamedEntries(BlogEntries):
    """Named container for a set of blog entries.
    """
    
    typename = None
    prefix = None
    
    sourcetype_attrname = None
    
    title_varnames = (
        'name',
    )
    
    heading_varnames = (
        'typename',
        'name'
    )
    
    default_title = "{name}"
    default_heading = "{typename}: {name}"
    
    def __init__(self, blog, name):
        BlogEntries.__init__(self, blog)
        self.name = self.sortkey = name
        if self.prefix:
            self.urlshort = "/{0}/{1}/".format(self.prefix, name)
        else:
            self.urlshort = "/{}/".format(name)
    
    @cached_property
    def prev_next_suffix(self):
        return self.name
    
    def _get_sourcetype_sources(self):
        if self.sourcetype_attrname:
            return sorted(getattr(self.blog, self.sourcetype_attrname), key=attrgetter('sortkey'), reverse=True)
        return BlogEntries._get_sourcetype_sources(self)


class EntryMixin(BlogConfigUser):
    """All entry mixins in extensions must subclass this class.
    """
    pass


class PageMixin(BlogConfigUser):
    """All page mixins in extensions must subclass this class.
    """
    pass


class BlogMixin(BlogConfigUser):
    """All blog mixins in extensions must subclass this class.
    """
    pass


class BlogExtensionMeta(BlogConfigUserMeta):
    """Metaclass to automatically detect mixins in extension modules.
    """
    
    def __init__(cls, name, bases, attrs):
        super(BlogExtensionMeta, cls).__init__(name, bases, attrs)
        # Auto-detect entry, page, and blog mixins in this class's module
        thismod = sys.modules[__name__]
        clsmod = sys.modules[cls.__module__]
        for etype in ('Entry', 'Page', 'Blog'):
            mixin_klass = getattr(thismod, '%sMixin' % etype)
            ext_mixin = first_subclass(clsmod, mixin_klass)
            if ext_mixin:
                setattr(cls, '%s_mixin' % etype.lower(), ext_mixin)


class BlogExtension(BlogConfigUser, metaclass=BlogExtensionMeta):
    """Base class for extension mechanism.
    """
    
    config_vars = dict(
        container_link_template='<a href="{urlshort}">{title}</a>',
        container_link_sep='',
    )
    
    def __init__(self, config):
        BlogConfigUser.__init__(self, config)
        
        # Register this extension in the appropriate places
        attr_tmpl = '{}_'
        mixin_tmpl = '{}_mixin'
        for etype in extension_types:
            # Check for methods that extend a known attribute
            if any(
                item for item in prefixed_items(dir(self), attr_tmpl.format(etype))
                if not item.endswith('mixin')
            ):
                extension_map[etype].append(self)
            # Check for extensions that declare mixins
            mixin = getattr(self, mixin_tmpl.format(etype), None)
            if mixin:
                mixin.extension_type = etype
                extend_attributes(mixin)
                oldcls = extension_types[etype]
                
                @wraps_class(oldcls)
                class Extended(mixin, oldcls):
                    pass
                
                extension_types[etype] = Extended
        
        # Allow for post-init processing in subclasses
        self.post_init()
    
    def post_init(self):
        pass
    
    def get_links(self, containers, reverse=False):
        """Return HTML links to containers.
        """
        return "{}{}".format(self.container_link_sep, newline).join(
            self.container_link_template.format(**c.link_attrs)
            for c in sorted(containers, key=attrgetter('sortkey'), reverse=reverse)
        )



#!/usr/bin/env python3
"""
Module RENDER_STATIC -- Simple Blog Static Rendering
Sub-Package SIMPLEBLOG.COMMANDS
Copyright (C) 2012-2013 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information
"""

import os

from plib.stdlib.ostools import data_changed

from simpleblog.commands import BlogCommand


class RenderStatic(BlogCommand):
    """Static rendering of all blog pages.
    """
    
    config_vars = dict(
        static_dir="static"
    )
    
    options = (
        ("-f", "--force", {
            'action': 'store_true',
            'help': "force writing of unchanged files"
        }),
        ("-q", "--quiet", {
            'action': 'store_true',
            'help': "suppress console output"
        })
    )
    
    def run(self, blog):
        for page in blog.pages:
            data = page.encoded
            path = os.path.abspath(os.path.join(self.static_dir, page.filepath))
            if self.opts.force or data_changed(data, path):
                if not self.opts.quiet:
                    print("Rendering", path)
                dir = os.path.split(path)[0]
                if not os.path.isdir(dir):
                    os.makedirs(dir)
                with open(path, 'wb') as f:
                    f.write(data)
            else:
                if not self.opts.quiet:
                    print(path, "is unchanged")



#!/usr/bin/env python3
"""
Module LOCALIZE -- Simple Blog Localization Extension
Sub-Package SIMPLEBLOG.EXTENSIONS
Copyright (C) 2012-2013 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information
"""

from simpleblog.extensions import BlogExtension


class LocalizeExtension(BlogExtension):
    """Allow metadata to specify localization for blog.
    """
    
    def blog_post_init(self, blog):
        try:
            blog.metadata.setdefault('locale', "{0}_{1}.{2}".format(
                blog.metadata['language'],
                blog.metadata['country'],
                blog.metadata['charset']
            ))
        except AttributeError:
            pass



#!/usr/bin/env python3
"""
Module QUOTE -- Simple Blog Quote Extension
Sub-Package SIMPLEBLOG.EXTENSIONS
Copyright (C) 2012-2013 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information
"""

from urllib.parse import quote_plus, urljoin

from plib.stdlib.iters import suffixed_items

from simpleblog.extensions import BlogExtension


class QuoteExtension(BlogExtension):
    """Add quoted versions of all URLs to blog metadata.
    """
    
    def blog_mod_required_metadata(self, blog, data):
        data.add(
            'root_url'
        )
        return data
    
    def blog_post_init(self, blog):
        urlkeys = set(suffixed_items(blog.metadata, '_url', full=True))
        tmpl = '{}_quoted'
        for key in urlkeys:
            newkey = tmpl.format(key)
            if newkey not in blog.metadata:
                url = blog.metadata[key]
                if key == 'root_url':
                    url = urljoin(url, "/index.html")
                else:
                    url = urljoin(blog.metadata['root_url'], blog.metadata[key])
                blog.metadata[newkey] = quote_plus(url)



#!/usr/bin/env python3
"""
Module TIMESTAMPS -- Simple Blog Timestamps Extension
Sub-Package SIMPLEBLOG.EXTENSIONS
Copyright (C) 2012-2013 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information
"""

from datetime import datetime

from simpleblog import extendable_property, extendable_method
from simpleblog.caching import cached
from simpleblog.extensions import BlogExtension, EntryMixin


timestamps_file = BlogExtension.config.get('timestamps_file', "timestamps")

timestamp_cache_format = "%Y-%m-%d %H:%M"


class TimestampEntryMixin(EntryMixin):
    
    @extendable_method()
    def string_from_datetime(self, dt, fmt):
        return dt.strftime(fmt)
    
    @extendable_property(
        cached(timestamps_file, reverse=True)
    )
    def timestamp_str(self):
        dt = self.datetime_from_mtime(self.mtime)
        return self.string_from_datetime(dt, timestamp_cache_format)
    
    @extendable_method()
    def datetime_from_string(self, s, fmt):
        return datetime.strptime(s, fmt)


class TimestampsExtension(BlogExtension):
    """Cache entry timestamps.
    """
    
    def entry_get_timestamp(self, entry):
        return entry.datetime_from_string(entry.timestamp_str, timestamp_cache_format)



#!/usr/bin/env python3
"""
Module TITLE -- Simple Blog Title Extension
Sub-Package SIMPLEBLOG.EXTENSIONS
Copyright (C) 2012-2013 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information
"""

import re

from plib.stdlib.decotools import cached_property

from simpleblog import extendable_property, newline
from simpleblog.caching import cached
from simpleblog.extensions import BlogExtension, EntryMixin


titles_file = BlogExtension.config.get('titles_file', "titles")


class TitleEntryMixin(EntryMixin):
    
    config_vars = dict(
        title_separator=newline,
        title_format=False
    )
    
    def _do_load(self):
        raw = super(TitleEntryMixin, self)._do_load()
        if self.title_separator in raw:
            self._titlestr, raw = raw.split(self.title_separator, 1)
        else:
            self._titlestr = ""
        return raw
    
    @cached_property
    def title_rexps(self):
        return [(re.compile(r), s) for r, s in [
            (r"\*\*([A-Za-z0-9]+)\*\*", "<strong>\g<1></strong>"),
            (r"\*([A-Za-z0-9]+)\*", "<em>\g<1></em>")
        ]]
    
    def _format_title(self):
        if self._titlestr and self.title_format:
            for rexp, repl in self.title_rexps:
                self._titlestr = rexp.sub(repl, self._titlestr)
        return self._titlestr
    
    @extendable_property(
        cached(titles_file)
    )
    def title(self):
        self.load()
        return self._format_title()


class TitleExtension(BlogExtension):
    """Parse entry title out of entry source.
    """
    pass



#!/usr/bin/env python3
"""
Module CATEGORIES -- Simple Blog Categories Extension
Sub-Package SIMPLEBLOG.EXTENSIONS
Copyright (C) 2012-2013 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information
"""

import os

from plib.stdlib.ostools import subdirs

from simpleblog import extendable_property
from simpleblog.extensions import BlogExtension, EntryMixin, NamedEntries


class BlogCategory(NamedEntries):
    """Category of blog entries.
    """
    
    config_vars = dict(
        prefix=('categories_prefix', "")
    )
    
    sourcetype = 'category'
    typename = "Category"
    sourcetype_attrname = 'all_categories'
    
    def _get_entries(self):
        return [
            e for e in self.blog.all_entries if e.category == self.name
        ]


class CategoryEntryMixin(EntryMixin):
    
    @extendable_property()
    def category(self):
        return self._category


class CategoryExtension(BlogExtension):
    """Add category to entry and category pages to blog.
    """
    
    config_vars = dict(
        category_link_template='<a href="/{category}/">{category}</a>',
        no_category_link="(None)"
    )
    
    def make_category_link(self, entry):
        return self.category_link_template.format(
            category=entry.category,
            **entry.metadata
        )
    
    def entry_get_name(self, entry):
        return entry._name
    
    def entry_post_init(self, entry):
        # The category will be an empty string for entries in
        # the root entries dir instead of a subdir
        entry._category, entry._name = os.path.split(entry.cachekey)
        
        # Use entry.category instead of entry._category here and elsewhere
        # in case the property is extended elsewhere
        if entry.category:
            link = self.make_category_link(entry)
        else:
            link = self.no_category_link
        entry.metadata.update(
            categorylink=link
        )
    
    def entry_mod_attrs(self, entry, attrs, format, params):
        attrs.update(
            category=entry.category
        )
        return attrs
    
    def blog_mod_all_entries(self, blog, all_entries):
        # Entries can now live in subdirs of entries_dir (each
        # subdir defines a category)
        return all_entries + [
            blog.entry_class(blog, os.path.join(subdir, name))
            for subdir in subdirs(blog.entries_dir)
            for name in blog.filter_entries(os.path.join(blog.entries_dir, subdir))
        ]
    
    def blog_mod_sources(self, blog, sources):
        
        blog.category_names = set(subdirs(blog.entries_dir))
        
        blog.all_categories = [
            BlogCategory(blog, catname)
            for catname in blog.category_names
        ]
        
        blog.metadata.update(
            category_links=self.get_links(blog.all_categories)
        )
        
        sources.extend(
            (category, "html")
            for category in blog.all_categories
        )
        
        return sources



#!/usr/bin/env python3
"""
Module RUN -- Simple Blog Command Runner
Package SIMPLEBLOG
Copyright (C) 2012-2013 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information
"""

from plib.stdlib.options import prepare_specs, update_parser, invoke_parser

from simpleblog import BlogError, load_blog
from simpleblog.commands import BlogCommand
from simpleblog.sub import load_sub


class BlogCommandError(BlogError):
    pass


def run(cmdname, parser, opts, goptlist, result=None, remaining=None):
    config, blog = load_blog(opts)
    
    mod, klass = load_sub(
        cmdname,
        "command", config.get('command_dir', ""),
        BlogCommandError, BlogCommand
    )
    
    optlist, arglist = prepare_specs(klass.options or (), klass.arguments or ())
    update_parser(parser, optlist, arglist)
    opts, args = invoke_parser(parser, goptlist + optlist, arglist, remaining, result)
    
    if opts.help:
        # Hack to make it look like the help is specific to cmdname; this is
        # *not* documented in the argparse module ;) (actually, we rely in
        # part on plib.stdlib.options setting the metavar keyword)
        for action in parser._actions:
            if action.metavar == "COMMAND":
                action.metavar = cmdname  # this displays the specific command name
                action.nargs = 1  # this removes the brackets from the command name
                # There can be only one
                break
        parser.print_help()
    
    else:
        cmd = klass(config, opts, args)
        cmd.run(blog)



#!/usr/bin/env python3
"""
Module FEED -- Simple Blog Feed Extension
Sub-Package SIMPLEBLOG.EXTENSIONS
Copyright (C) 2012-2013 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information
"""

import re
from datetime import datetime
from itertools import groupby

from plib.stdlib.decotools import cached_property, cached_method
from plib.stdlib.localize import weekdayname, monthname, monthname_long
from plib.stdlib.tztools import UTCTimezone, LocalTimezone

from simpleblog import extendable_property, BlogEntries, newline
from simpleblog.extensions import BlogExtension, BlogMixin, EntryMixin


tz_utc = UTCTimezone()

tz_local = LocalTimezone()


archive_marker = "<fh:archive />"

archive_current_tmpl = '<link rel="current" href="{}/index.{}" />'

archive_rel_tmpl = '<link rel="{}-archive" href="{}{}{}.{}" />'

archive_rel_specs = ('prev', 'next')


class BlogCurrentFeedEntries(BlogEntries):
    """Current syndication feed entries.
    """
    
    config_vars = dict(
        archive_use_monthnames=False,
        archive_long_monthnames=False,
        archive_feed_dirs=False
    )
    
    urlshort = "/"
    
    heading_varnames = (
        'title',
    )
    
    default_heading = "Feed Archive: {title}"
    
    is_current_feed = True
    
    def __init__(self, blog, arglist, *args):
        BlogEntries.__init__(self, blog)
        self.arglist = arglist
        self.args = args
        self.argindex = i = arglist.index(args)
        
        assert self.is_current_feed == (i == (len(arglist) - 1))
        
        self.prev_args = arglist[i - 1] if i > 0 else None
        self.next_args = arglist[i + 1] if i < (len(arglist) - 2) else None
        
        self.default_title = self.argstr('-', *args)
    
    @cached_method
    def argstr(self, sep, *args):
        if (len(args) > 1) and self.archive_use_monthnames:
            arg1 = (
                monthname_long(args[1]) if self.archive_long_monthnames
                else monthname(args[1])
            )
            args = (args[0], arg1) + args[2:]
        return sep.join(
            str(arg).rjust(2, '0') for arg in args
        )
    
    @cached_method
    def entry_match(self, entry):
        t = entry.timestamp
        return (t.year, t.month, t.day)[:len(self.args)] == self.args
    
    def _get_entries(self):
        for entry in self.blog.all_entries:
            if self.entry_match(entry):
                yield entry
    
    @cached_method
    def args_urlshort(self, *args):
        if self.archive_feed_dirs:
            sep = suffix = '/'
        else:
            sep = '-'
            suffix = ''
        return "/{}{}".format(self.argstr(sep, *args), suffix)
    
    @cached_property
    def archive_elem(self):
        return "" if self.is_current_feed else archive_marker
    
    @cached_method
    def archive_current(self, format):
        return archive_current_tmpl.format(
            self.blog.metadata['root_url'],
            format
        ) if not self.is_current_feed else ""
    
    @cached_method
    def archive_rel(self, rel, format):
        args = getattr(self, '{}_args'.format(rel), None)
        if args:
            return archive_rel_tmpl.format(
                rel,
                self.blog.metadata['root_url'],
                self.args_urlshort(*args),
                "index" if self.archive_feed_dirs else "",
                format
            )
        return ""
    
    @cached_method
    def archive_elements(self, format):
        return "{}{}{}".format(
            newline,
            newline.join(
                item for item in [
                    self.archive_elem,
                    self.archive_current(format)
                ] + [
                    self.archive_rel(rel, format)
                    for rel in archive_rel_specs
                ]
                if item
            ),
            newline
        )


class BlogArchiveFeedEntries(BlogCurrentFeedEntries):
    """Feed archive entries.
    """
    
    is_current_feed = False
    
    @cached_property
    def urlshort(self):
        return self.args_urlshort(*self.args)
    
    def _get_urlpath(self):
        if (not self.archive_feed_dirs) and (self.urlshort is not None):
            return self.urlshort
        return super(BlogArchiveFeedEntries, self)._get_urlpath()


template_rss = "{0}, {1.day:02d} {2} {1.year} {1.hour:02d}:{1.minute:02d} GMT"

template_atom = "{0.year}-{0.month:02d}-{0.day:02d}T{0.hour:02d}:{0.minute:02d}:00Z"


def rss_format(t):
    return template_rss.format(
        weekdayname(t.weekday(), dt=True),
        t,
        monthname(t.month)
    )


def atom_format(t):
    return template_atom.format(t)


class FeedEntryMixin(EntryMixin):
    
    config_vars = dict(
        utc_timestamps=False
    )
    
    @extendable_property()
    def timestamp_utc(self):
        t = self.timestamp
        if self.utc_timestamps:
            return datetime(t.year, t.month, t.day, t.hour, t.minute, t.second, tzinfo=tz_utc)
        return datetime(t.year, t.month, t.day, t.hour, t.minute, t.second, tzinfo=tz_local).astimezone(tz_utc)
    
    @extendable_property()
    def timestamp_atom(self):
        return atom_format(self.timestamp_utc)
    
    @extendable_property()
    def timestamp_rss(self):
        return rss_format(self.timestamp_utc)


known_feed_formats = ["rss", "atom"]

known_archive_feed_formats = ["atom"]


class FeedBlogMixin(BlogMixin):
    
    @extendable_property()
    def feed_formats(self):
        return set(known_feed_formats).intersection(self.index_formats)
    
    @extendable_property()
    def archive_feed_formats(self):
        return set(known_archive_feed_formats).intersection(self.index_formats)
    
    @cached_property
    def feedlink_template_rss(self):
        return self.template_data("link", "rss")
    
    @cached_property
    def feedlink_template_atom(self):
        return self.template_data("link", "atom")


re_link = re.compile(r'<a href=\"\/([A-Za-z0-9\-\/\.]+)\"')


def fixup_relative_links(html, root_url):
    """Convert relative links in HTML to absolute links based on root URL.
    """
    return re.sub(
        re_link,
        lambda m: '<a href="{0}/{1}"'.format(root_url, m.group(1)),
        html
    )


class FeedExtension(BlogExtension):
    """Add RSS and Atom feed links to home page.
    """
    
    config_vars = dict(
        archive_feeds=None,
        atom_id_template="{cachekey}",
        atom_category_template="entries",
        rss_id_template="{cachekey}",
        rss_category_template="entries"
    )
    
    def entry_mod_body(self, entry, body, format, params):
        if format in entry.blog.feed_formats:
            return fixup_relative_links(body, entry.blog.metadata['root_url'])
        return body
    
    def entry_mod_attrs(self, entry, attrs, format, params):
        do_rss = 'rss' in entry.blog.feed_formats
        do_atom = 'atom' in entry.blog.feed_formats
        if do_rss or do_atom:
            attrs.update(
                root_url=entry.blog.metadata['root_url'],
                author=entry.blog.metadata['author'],
                email=entry.blog.metadata['email']
            )
            if do_rss:
                attrs.update(
                    id_rss=self.rss_id_template.format(**attrs),
                    category_rss=self.rss_category_template.format(**attrs),
                    timestamp_rss=entry.timestamp_rss
                )
            if do_atom:
                attrs.update(
                    id_atom=self.atom_id_template.format(**attrs),
                    category_atom=self.atom_category_template.format(**attrs),
                    timestamp_atom=entry.timestamp_atom
                )
        return attrs
    
    def page_mod_attrs(self, page, attrs):
        if page.format in page.blog.feed_formats:
            t = max(entry.timestamp_utc for entry in page.entries)
            if page.format in page.blog.archive_feed_formats:
                attrs.update(
                    page_archive_elements=page.source.archive_elements(
                        page.format
                    ) if self.archive_feeds else ""
                )
            if page.format == 'atom':
                attrs.update(
                    page_timestamp_atom=atom_format(t)
                )
            if page.format == 'rss':
                language = page.blog.metadata['language']
                try:
                    country = page.blog.metadata['country'].lower()
                except KeyError:
                    pass
                else:
                    language = '-'.join([language, country])
                attrs.update(
                    page_timestamp_rss=rss_format(t),
                    blog_language_rss=language
                )
        return attrs
    
    def blog_mod_required_metadata(self, blog, data):
        data.update([
            'root_url',
            'author',
            'email'
        ])
        if 'rss' in blog.feed_formats:
            data.add('language')
        return data
    
    def blog_mod_default_metadata(self, blog, data):
        if 'rss' in blog.feed_formats:
            data.update(
                rss_title="RSS",
                rss_url="/index.rss"
            )
        if 'atom' in blog.feed_formats:
            data.update(
                atom_title="Atom",
                atom_url="/index.atom",
            )
        return data
    
    @cached_method
    def archive_feed_args(self, blog):
        eattrs = ('year', 'month', 'day')
        eattrs = eattrs[:eattrs.index(self.archive_feeds) + 1]
        return [
            groupkey for groupkey, _ in groupby(
                sorted(e.timestamp for e in blog.all_entries),
                lambda t: tuple(
                    getattr(t, attrname) for attrname in eattrs
                )
            )
        ]
    
    @cached_method
    def current_feed_entries(self, blog):
        arglist = self.archive_feed_args(blog)
        return BlogCurrentFeedEntries(blog, arglist, *arglist[-1])
    
    @cached_method
    def archive_feed_entries(self, blog, *args):
        arglist = self.archive_feed_args(blog)
        return BlogArchiveFeedEntries(blog, arglist, *args)
    
    def blog_mod_index_entries(self, blog, entries, format):
        if self.archive_feeds and (format in blog.archive_feed_formats):
            return self.current_feed_entries(blog)
        return entries
    
    def blog_mod_sources(self, blog, sources):
        
        feedlinks = []
        if 'rss' in blog.feed_formats:
            feedlinks.append(
                blog.feedlink_template_rss.format(**blog.metadata)
            )
        if 'atom' in blog.feed_formats:
            feedlinks.append(
                blog.feedlink_template_atom.format(**blog.metadata)
            )
        blog.metadata.update(
            feed_links=newline.join(feedlinks)
        )
        
        if self.archive_feeds:
            arglist = self.archive_feed_args(blog)
            sources.extend(
                (self.archive_feed_entries(blog, *args), format)
                for args in arglist[:-1]
                for format in blog.archive_feed_formats
            )
        
        return sources



#!/usr/bin/env python3
"""
Module EXT -- Simple Blog Extension Loader
Package SIMPLEBLOG
Copyright (C) 2012-2013 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information
"""

from simpleblog import BlogError
from simpleblog.extensions import BlogExtension
from simpleblog.sub import load_subs


class BlogExtensionError(BlogError):
    pass


def load(config, extensions):
    # This hack is useful for extensions that need to access the
    # config in module or class level code, e.g., in decorators
    BlogExtension.config = config
    exts = load_subs(
        extensions,
        "extension", config.get('extension_dir', ""),
        BlogExtensionError, BlogExtension
    )
    for mod, klass in exts:
        mod.extension = klass(config)



#!/usr/bin/env python3
"""
Module FOLDING -- Simple Blog Folding Extension
Sub-Package SIMPLEBLOG.EXTENSIONS
Copyright (C) 2012-2013 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information
"""

from plib.stdlib.decotools import cached_method

from simpleblog import (
    extendable_property, extendable_method,
    noresult, newline)
from simpleblog.extensions import BlogExtension, EntryMixin


class FoldEntryMixin(EntryMixin):
    
    config_vars = dict(
        fold_symbol="<!-- FOLD -->",
        fold_inline=False,
        short_formats=dict(
            vartype=set,
            default=["html"])
    )
    
    @extendable_property()
    def fold_marker(self):
        if self.fold_inline:
            return self.fold_symbol
        return '{0}{1}'.format(self.fold_symbol, newline)
    
    def _do_load(self):
        raw = super(FoldEntryMixin, self)._do_load()
        if self.fold_marker in raw:
            short, src = raw.split(self.fold_marker, 1)
            self._short = short
            raw = ''.join((short, src))
        else:
            self._short = None
        return raw
    
    @cached_method
    def has_short(self, format):
        self.load()
        if format in self.short_formats:
            try:
                return self._short is not None
            except AttributeError:
                pass
        return False
    
    @extendable_method()
    def short_template(self, format):
        if format in self.short_formats:
            return self.template_data("short", format)
        raise NotImplementedError
    
    @extendable_property()
    def rendered_short(self):
        self.load()
        return self.render(self._short)


class FoldingExtension(BlogExtension):
    """Generate short version of entry for index pages.
    """
    
    config_vars = dict(
        max_index_full=dict(
            vartype=lambda n: max(n - 1, 0),
            varkey='max_full_entries',
            default=1)
    )
    
    def page_mod_entry_params(self, page, params, entry):
        params.update(
            index=page.entries.index(entry)
        )
        return params
    
    def use_short_entry(self, entry, params):
        return (
            # This first clause allows other extensions to force a short entry
            params.get('force_short', False)
            or (params.index > self.max_index_full)
        )
    
    def entry_get_body(self, entry, format, params):
        if self.use_short_entry(entry, params) and entry.has_short(format):
            return entry.short_template(format).format(
                body=entry.rendered_short,
                link=entry.make_permalink(format)
            )
        # If no short form, don't override
        return noresult



#!/usr/bin/env python3
"""
Module PUBLISH -- Simple Blog Publisher
Sub-Package SIMPLEBLOG.COMMANDS
Copyright (C) 2012-2013 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information
"""

import os

from plib.stdlib.proc import process_call

from simpleblog.commands import BlogCommand


class Publish(BlogCommand):
    """Publish blog.
    """
    
    config_vars = dict(
        static_dir="static",
        publish_cmd_name="rsync",
        publish_cmd_options="-rt",
        ssh_user="",
        ssh_host="",
        ssh_path="~/",
    )
    
    options = (
        ("-q", "--quiet", {
            'action': 'store_true',
            'help': "suppress console output"
        }),
        ("-d", "--debug", {
            'action': 'store_true',
            'help': "print rsync command line for debugging"
        })
    )
    
    def run(self, blog):
        cmdline = ' '.join([
            self.publish_cmd_name,
            self.publish_cmd_options,
            os.path.join(os.path.abspath(self.static_dir), "*"),
            "{}@{}:{}".format(self.ssh_user, self.ssh_host, self.ssh_path)
        ])
        if self.opts.debug:
            print(cmdline)
        returncode, output = process_call(cmdline, shell=True)
        if (returncode != 0) or not self.opts.quiet:
            print(output.decode())
        if returncode != 0:
            print("Publish failed with return code {}".format(returncode))



#!/usr/bin/env python3
"""
Module SUB -- Simple Blog Sub-Package Handler
Package SIMPLEBLOG
Copyright (C) 2012-2013 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information
"""

from importlib import import_module

from plib.stdlib.classtools import first_subclass
from plib.stdlib.systools import tmp_sys_path


def load_submodule(subtype, subdir, name, err, subcls):
    name = name.replace('-', '_')
    mod = None
    if subdir:
        # User-supplied module takes precedence
        try:
            mod = import_module(name)
        except ImportError:
            pass
    if mod is None:
        try:
            mod = import_module("simpleblog.{0}s.{1}".format(subtype, name))
        except ImportError:
            raise err("{0} {1} not found!".format(subtype, name))
    klass = first_subclass(mod, subcls)
    if not klass:
        raise err("no {0} in {1} module!".format(subtype, name))
    return mod, klass


def load_sub(name, subtype, subdir, err, subcls):
    with tmp_sys_path(subdir):
        return load_submodule(subtype, subdir, name, err, subcls)


def load_subs(subs, subtype, subdir, err, subcls):
    with tmp_sys_path(subdir):
        for name in subs:
            yield load_submodule(subtype, subdir, name, err, subcls)



#!/usr/bin/env python3
"""
Module SERVE_LOCAL -- Simple Blog Test Server
Sub-Package SIMPLEBLOG.COMMANDS
Copyright (C) 2012-2013 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information
"""

import os
import http.server

from plib.stdlib.ostools import tmp_chdir

from simpleblog.commands import BlogCommand


class ServeLocal(BlogCommand):
    """Serve blog on localhost for testing.
    """
    
    config_vars = dict(
        static_dir="static"
    )
    
    options = (
        ("-n", "--hostname", {
            'action': 'store', 'type': str,
            'default': "localhost",
            'help': "host name or IP address"
        }),
        ("-p", "--port", {
            'action': 'store', 'type': int,
            'default': 8000,
            'help': "port number"
        }),
        ("-q", "--quiet", {
            'action': 'store_true',
            'help': "suppress console output"
        }),
    )
    
    def run(self, blog):
        http_root = os.path.abspath(self.static_dir)
        server_class = http.server.HTTPServer
        handler_class = http.server.SimpleHTTPRequestHandler
        server_address = (self.opts.hostname, self.opts.port)
        if not self.opts.quiet:
            print("Serving files under {} at {}".format(
                http_root,
                "http://{}:{:d}/".format(self.opts.hostname, self.opts.port)
            ))
        httpd = server_class(server_address, handler_class)
        with tmp_chdir(http_root):
            try:
                httpd.serve_forever()
            except KeyboardInterrupt:
                if not self.opts.quiet:
                    print("Shutting down....")



#!/usr/bin/env python3
"""
Sub-Package SIMPLEBLOG.COMMANDS
Copyright (C) 2012-2013 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information
"""

from simpleblog import BlogConfigUser


class BlogCommand(BlogConfigUser):
    """Base class for command mechanism.
    """
    
    options = None
    arguments = None
    
    def __init__(self, config, opts, args):
        BlogConfigUser.__init__(self, config)
        self.opts = opts
        self.args = args
    
    def run(self, blog):
        raise NotImplementedError



#!/usr/bin/env python3
"""
Module LINKS -- Simple Blog Entry Links Extension
Sub-Package SIMPLEBLOG.EXTENSIONS
Copyright (C) 2012-2013 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information
"""

from itertools import groupby
from operator import itemgetter

from plib.stdlib.decotools import cached_property, cached_method

from simpleblog import extendable_method, newline
from simpleblog.extensions import BlogExtension, EntryMixin


class LinksEntryMixin(EntryMixin):
    
    config_vars = dict(
        entrylink_sep="&nbsp;",
        link_next_template="next in {}",
        link_next_title_template="Next in {}",
        link_prev_template="previous in {}",
        link_prev_title_template="Previous in {}",
        link_display_sourcetypes=dict(
            vartype=set,
            default=['entry'])
    )
    
    @cached_property
    def entrylinks_template(self):
        return self.template_data("entry", "links")
    
    @cached_method
    def get_entrylink(self, attr, format, prefix):
        entry, label = attr
        tmpl_title, tmpl_content = (
            (self.link_prev_title_template, self.link_prev_template)
            if prefix.startswith('p') else
            (self.link_next_title_template, self.link_next_template)
        )
        content = tmpl_content.format(label)
        if entry is not None:
            title = tmpl_title.format(label.capitalize()) if tmpl_title else None
            href = entry.make_permalink(format)
            if title:
                return '<a href="{}" title="{}">{}</a>'.format(href, title, content)
            return '<a href="{}">{}</a>'.format(href, content)
        return content
    
    @extendable_method()
    def prev_next_link(self, attr, format, prefix):
        if isinstance(attr, list):
            sep = '{0}{1}{0}'.format(newline, self.entrylink_sep)
            return sep.join(
                self.get_entrylink(value, format, prefix)
                for value in attr
            )
        return self.get_entrylink(attr, format, prefix)
    
    @extendable_method()
    def prev_next_attrs(self, format):
        prefixes = ('next_in_', 'prev_in_')
        return dict(
            (attrname, self.prev_next_link(
                getattr(self, attrname), format, prefix
            ))
            for prefix in prefixes
            for attrname in dir(self)
            if attrname.startswith(prefix)
        )
    
    @cached_method
    def make_entrylinks(self, format, params):
        if params.sourcetype in self.link_display_sourcetypes:
            return self.entrylinks_template.format(**self.prev_next_attrs(format))
        return ""


class LinksExtension(BlogExtension):
    """Add links to previous and next entries in containers.
    """
    
    config_vars = dict(
        link_sourcetypes=dict(
            blog=None
        )
    )
    
    def entry_mod_attrs(self, entry, attrs, format, params):
        attrs.update(
            entrylinks=entry.make_entrylinks(format, params)
        )
        return attrs
    
    def page_mod_entry_params(self, page, params, entry):
        params.update(
            sourcetype=page.source.sourcetype
        )
        return params
    
    def page_mod_attrs(self, page, attrs):
        attrs.update(
            page_entrylinks=(
                page.source.make_entrylinks(page.format, page.entry_params(page.source))
                if page.source and (page.source.sourcetype == 'entry')
                else ""
            )
        )
        return attrs
    
    def blog_mod_sources(self, blog, sources):
        prev_tmpl = 'prev_in_{}'
        next_tmpl = 'next_in_{}'
        
        # Set up previous/next links within each source
        multisources = set()
        for source, _ in groupby(sources, itemgetter(0)):
            sourcetype = source.sourcetype
            if sourcetype in self.link_sourcetypes:
                if source.multisource:
                    multisources.add(source.multisource)
                propname = self.link_sourcetypes[sourcetype]
                prop = getattr(source, propname) if propname else None
                entries = source.entries
                for index, entry in enumerate(entries):
                    prev_attr = (
                        entries[index - 1]
                        if index > 0 else None,
                        prop or sourcetype
                    )
                    next_attr = (
                        entries[index + 1]
                        if index < (len(entries) - 1) else None,
                        prop or sourcetype
                    )
                    if source.entry_sort_reversed:
                        prev_attr, next_attr = next_attr, prev_attr
                    if not source.multisource:
                        setattr(entry, prev_tmpl.format(sourcetype), prev_attr)
                        setattr(entry, next_tmpl.format(sourcetype), next_attr)
                    if prop:
                        setattr(entry, prev_tmpl.format(prop), prev_attr)
                        setattr(entry, next_tmpl.format(prop), next_attr)
        
        # Add previous/next link collections for multisources
        for entry in blog.all_entries:
            for attrname in multisources:
                names = sorted(getattr(entry, attrname))
                if names:
                    setattr(entry, prev_tmpl.format(attrname), [
                        getattr(entry, prev_tmpl.format(name))
                        for name in names
                    ])
                    setattr(entry, next_tmpl.format(attrname), [
                        getattr(entry, next_tmpl.format(name))
                        for name in names
                    ])
        
        return sources



#!/usr/bin/env python3
"""
Package SIMPLEBLOG -- Simple Python blog system
Copyright (C) 2012-2013 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information
"""

import os
import pkgutil
from codecs import decode, encode
from collections import defaultdict
from datetime import datetime
from functools import wraps
from operator import attrgetter

from plib.stdlib.decotools import (
    cached_method, cached_property, memoize_generator)
from plib.stdlib.ini import PIniFile
from plib.stdlib.ini.defs import *
from plib.stdlib.iters import suffixed_items
from plib.stdlib.localize import (
    weekdayname, weekdayname_long,
    monthname, monthname_long)


__version__ = "0.9.2"


blogfile_exts = ["json"]

try:
    from yaml import load as _loads
except ImportError:
    from json import loads
else:
    blogfile_exts.insert(0, "yaml")
    
    from functools import partial
    
    try:
        # Use libyaml if present, much faster
        from yaml import CSafeLoader as _Loader
    except ImportError:
        from yaml import SafeLoader as _Loader
    
    # Make PyYAML load unicode strings
    _Loader.add_constructor(
        'tag:yaml.org,2002:str',
        lambda self, node: self.construct_scalar(node)
    )
    
    loads = partial(_loads, Loader=_Loader)


inifile = PIniFile("simpleblog", [
    ('source', [
        ('encoding', INI_STRING, 'utf-8')
    ])
])


def blogdata(data, encoding=inifile.source_encoding):
    return decode(data, encoding)


def read_blogfile(filename):
    with open(filename, 'rb') as f:
        data = f.read()
    return blogdata(data)


def load_blogfile(filename, basename, mapping):
    trial_names = [filename] + [
        "{0}.{1}".format(basename, ext)
        for ext in blogfile_exts
    ]
    for fname in trial_names:
        if fname and os.path.isfile(fname):
            mapping.update(loads(read_blogfile(fname)))
            break


# CONFIG

newline = os.linesep


class BlogError(Exception):
    pass


class BlogConfigError(BlogError):
    pass


class BlogConfig(object):
    """Blog configuration.
    """
    
    def __init__(self, filename=None):
        self.filename = filename
        self.settings = {}
        load_blogfile(self.filename, "config", self.settings)
        
        # This will prove to be convenient
        self.get = self.settings.get
    
    def __getattr__(self, name):
        try:
            return self.settings[name]
        except KeyError:
            raise AttributeError


def make_config_property(key, value):
    """Return``cached_property`` for config setting.
    
    The ``key`` argument is the name of the property.
    
    The ``value`` argument is either a default value, or a
    2-tuple ``(varkey, default)`` giving the key of the setting
    in the config and the default value, or a dict with one or
    more of the following keys: ``varkey``, ``default``,
    ``vartype``, where ``vartype`` gives a type to which the
    returned value will be coerced.
    """
    
    if isinstance(value, dict):
        vartype = value.get('vartype', None)
        varkey = value.get('varkey', key)
        default = value.get('default', None)
    elif isinstance(value, tuple):
        vartype = None
        varkey, default = value
    else:
        vartype = None
        varkey = key
        default = value
    if vartype is not None:
        def fget(self):
            return vartype(self.config.get(varkey, default))
    else:
        def fget(self):
            return self.config.get(varkey, default)
    fget.__name__ = key
    return cached_property(fget)


class BlogConfigUserMeta(type):
    """Metaclass to automatically set up configurable attributes.
    
    We do this with a metaclass instead of a class decorator because
    we want every subclass of ``BlogConfigUser`` to have configurable
    attributes automatically set up, without requiring the programmer
    to remember to decorate the class.
    """
    
    def __init__(cls, name, bases, attrs):
        super(BlogConfigUserMeta, cls).__init__(name, bases, attrs)
        # Only do this for config vars declared in this class
        for key, value in attrs.get('config_vars', {}).items():
            # The make_config_property function is factored out
            # to ensure each closure it returns is "clean"
            setattr(cls, key, make_config_property(key, value))


class BlogConfigUser(object, metaclass=BlogConfigUserMeta):
    """Base class for objects that use a config.
    """
    
    def __init__(self, config):
        self.config = config


# BASE


class BlogTemplateError(BlogError):
    pass


class BlogObject(BlogConfigUser):
    """Base object for items managed by blog.
    """
    
    config_vars = dict(
        entries_dir="entries",
        entry_ext=".html",
        template_dir="templates"
    )
    
    def __init__(self, blog):
        self.blog = blog
        self.config = self.blog.config  # don't need to call the superclass __init__
    
    @cached_method
    def template_basename(self, kind, format):
        return "{0}.{1}".format(kind, format)
    
    @cached_method
    def template_file(self, kind, format):
        return os.path.join(self.template_dir, self.template_basename(kind, format))
    
    @cached_method
    def template_data(self, kind, format):
        try:
            return read_blogfile(self.template_file(kind, format))
        except IOError:
            try:
                return blogdata(pkgutil.get_data(
                    'simpleblog',
                    "templates/{}".format(self.template_basename(kind, format))
                ))
            except IOError:
                raise BlogConfigError("template {}.{} not found".format(kind, format))


extension_types = {}

extension_map = defaultdict(list)

noresult = object()  # marker for no result returned

noreturn = object()  # marker for no return value at all


def check_extensions(instance, etype, key,
                     args=None, kwargs=None, result=noresult, modifying=False):
    args = args or ()
    kwargs = kwargs or {}
    for extension in extension_map.get(etype, ()):
        ext = getattr(extension, key, None)
        if ext is not None:
            if result is noreturn:
                ext(instance, *args, **kwargs)
            elif modifying:
                result = ext(instance, result, *args, **kwargs)
            else:
                result = ext(instance, *args, **kwargs)
            if (not modifying) and (result not in (noresult, noreturn)):
                break
    return result


class extendable_attr(object):
    """Base class for attributes that will use the extension mechanism.
    
    An instance of this class is applied as a decorator to mark a
    property or method of an extendable class that will use the
    extension mechanism. The instance wraps the function that
    implements the base property or method when it is called as
    a decorator. Arguments to the decorator give decorators that
    will be applied to the fully implemented property or method.
    Note that even if there are no arguments, the decorator must
    be applied with a call, like so:
    
    @extendable_property()
    def prop(self):
        ...
    """
    
    attrtype = None
    
    def __init__(self, *decorators):
        self._decorators = decorators
    
    def __call__(self, func):
        # What happens when this class is called as a decorator
        self._func = func
        return self
    
    def make_method(self, etype, name):
        assert name == self._func.__name__
        getkey = '{0}_get_{1}'.format(etype, name)
        modkey = '{0}_mod_{1}'.format(etype, name)
        
        @wraps(self._func)
        def meth(innerself, *args, **kwargs):
            # Check for extensions that might override the original function
            result = check_extensions(innerself, etype, getkey, args, kwargs)
            # Only call original function if no extension overrode it
            if result is noresult:
                result = self._func(innerself, *args, **kwargs)
            # Now check for extensions that want to modify the result
            result = check_extensions(
                innerself, etype, modkey, args, kwargs,
                result, True
            )
            return result
        
        return meth
    
    def as_extended(self, cls, name):
        meth = self.make_method(cls.extension_type, name)
        for d in reversed(self._decorators):
            # Reversing makes the declaration order more intuitive
            meth = d(meth)
        return self.attrtype(meth)


class extendable_property(extendable_attr):
    """Property to be fully implemented by ``extendable`` class decorator.
    """
    
    attrtype = cached_property


class extendable_method(extendable_attr):
    """Method to be fully implemented by ``extendable`` class decorator.
    """
    
    attrtype = cached_method


def extend_attributes(cls):
    """Implement all extendable attributes in cls.
    """
    
    for key in dir(cls):
        attr = getattr(cls, key)
        if isinstance(attr, extendable_attr):
            setattr(cls, key, attr.as_extended(cls, key))


def extendable(cls):
    """Class decorator to implement extendable properties.
    """
    
    # Determine extension type and add to map
    etype = cls.__name__.lower()
    eprefix = 'blog'
    if len(etype) > len(eprefix):
        etype = etype[len(eprefix):]
    cls.extension_type = etype
    extension_types[etype] = cls
    
    # Implement all extendable attributes
    extend_attributes(cls)
    
    # Implement post_init call at end of constructor
    initkey = '{}_post_init'.format(etype)
    oldinit = cls.__init__
    
    @wraps(oldinit)
    def _newinit(self, *args, **kwargs):
        oldinit(self, *args, **kwargs)
        check_extensions(self, etype, initkey, result=noreturn)
    
    cls.__init__ = _newinit
    
    return cls


# ENTRY


class BlogSource(BlogObject):
    """Base class for blog objects that can be page sources.
    """
    
    sourcetype = None
    multisource = None
    
    @cached_property
    def next_source(self):
        return self._get_next_source()
    
    def _get_next_source(self):
        return None
    
    @cached_property
    def prev_source(self):
        return self._get_prev_source()
    
    def _get_prev_source(self):
        return None


@extendable
class BlogEntry(BlogSource):
    """Single blog entry.
    """
    
    config_vars = dict(
        utc_timestamps=False,
        timestamp_template="{hour:02d}:{minute:02d}",
        datestamp_template="{year}-{month:02d}-{day:02d}"
    )
    
    sourcetype = 'entry'
    
    def __init__(self, blog, name):
        BlogObject.__init__(self, blog)
        self.cachekey = name
        self.metadata = {}
    
    @extendable_property()
    def name(self):
        return self.cachekey
    
    @extendable_property()
    def urlpath(self):
        return "/{}".format(self.cachekey)
    
    @extendable_property()
    def heading(self):
        return "Single Entry"
    
    @extendable_property()
    def title(self):
        return self.name
    
    @extendable_property()
    def filename(self):
        return os.path.join(self.entries_dir, self.cachekey + self.entry_ext)
    
    # Note that mtime is *not* an extendable property; it must always represent
    # the last modification time of the entry's source, in file mtime (i.e.,
    # POSIX mtime) format. Mixins should override _get_mtime instead.
    
    @cached_property
    def mtime(self):
        return self._get_mtime()
    
    def _get_mtime(self):
        return os.path.getmtime(self.filename)
    
    @extendable_method()
    def datetime_from_mtime(self, mtime):
        tf = datetime.utcfromtimestamp if self.utc_timestamps else datetime.fromtimestamp
        return tf(mtime)
    
    @extendable_property()
    def timestamp(self):
        return self.datetime_from_mtime(self.mtime)
    
    @extendable_property()
    def timestamp_attrfuncs(self):
        return dict(
            year=None,
            month=None,
            day=None,
            hour=None,
            minute=None,
            second=None,
            weekdayname=lambda t: weekdayname(t.weekday(), dt=True),
            weekdayname_long=lambda t: weekdayname_long(t.weekday(), dt=True),
            monthname=lambda t: monthname(t.month),
            monthname_long=lambda t: monthname_long(t.month)
        )
    
    @extendable_method()
    def timestamp_attrs(self, dt):
        return dict(
            (key, (varfunc or attrgetter(key))(dt))
            for key, varfunc in self.timestamp_attrfuncs.items()
        )
    
    @extendable_property()
    def timestamp_vars(self):
        return self.timestamp_attrs(self.timestamp)
    
    @extendable_property()
    def timestamp_formatted(self):
        return self.timestamp_template.format(**self.timestamp_vars)
    
    @extendable_property()
    def datestamp_formatted(self):
        return self.datestamp_template.format(**self.timestamp_vars)
    
    # Note that source is *not* an extendable property! It must
    # always be the actual source loaded from the filesystem,
    # or whatever other source is being used. Mixins should
    # override _get_source instead, to ensure that the caching
    # mechanism works properly.
    
    @cached_property
    def source(self):
        return self._get_source()
    
    def _get_source(self):
        return read_blogfile(self.filename)
    
    # Load is also *not* extendable, because it must be callable
    # from any code that needs to ensure that the actual data is
    # loaded. Mixins should override _do_load.
    
    @cached_method
    def load(self):
        """Load raw data.
        """
        return self._do_load()
    
    def _do_load(self):
        return self.source
    
    # Render is not extendable because, like load, it needs to be
    # callable from any code that needs to render raw data. Mixins
    # should override _do_render.
    
    @cached_method
    def render(self, rawdata):
        """Convert raw data into rendered data.
        """
        return self._do_render(rawdata)
    
    def _do_render(self, rawdata):
        return rawdata
    
    @extendable_property()
    def rendered(self):
        return self.render(self.load())
    
    @extendable_method()
    def body(self, format, params):
        return self.rendered
    
    @extendable_method()
    def template(self, format):
        return self.template_data("entry", format)
    
    @cached_method
    def make_permalink(self, format):
        return "{}.{}".format(self.urlpath, format)
    
    @extendable_method()
    def attrs(self, format, params):
        return dict(
            self.timestamp_vars,
            name=self.name,
            cachekey=self.cachekey,
            urlpath=self.urlpath,
            title=self.title,
            body=self.body(format, params),
            timestamp=self.timestamp_formatted,
            datestamp=self.datestamp_formatted,
            permalink=self.make_permalink(format),
            **self.metadata
        )
    
    @extendable_method()
    def formatted(self, format, params):
        return self.template(format).format(**self.attrs(format, params))


# CONTAINERS


def make_config_or_default_property(key):
    """Return``cached_property`` for config_or_default setting.
    
    The ``key`` argument is the name of the property.
    """
    
    def fget(self):
        return self.config_or_default(key)
    fget.__name__ = key
    return cached_property(fget)


class BlogEntriesMeta(BlogConfigUserMeta):
    """Metaclass to automatically set up configs for blog entry containers.
    """
    
    def __init__(cls, name, bases, attrs):
        super(BlogEntriesMeta, cls).__init__(name, bases, attrs)
        # Similar setup to BlogConfigUserMeta, but for config_or_default_vars
        for key, value in attrs.get('config_or_default_vars', {}).items():
            setattr(cls, 'default_{}'.format(key), value)
            setattr(cls, key, make_config_or_default_property(key))


class BlogEntries(BlogSource, metaclass=BlogEntriesMeta):
    """Container for a set of blog entries.
    """
    
    config_vars = dict(
        entry_sort_key='timestamp',
        entry_sort_reversed=True
    )
    
    config_or_default_vars = dict(
        title="",
        heading=""
    )
    
    urlshort = ""
    
    @cached_method
    def config_or_default(self, key):
        default_tmpl = getattr(self, 'default_{}'.format(key))
        tmpl = self.config.get(
            '{}_{}_template'.format(self.sourcetype, key),
            default_tmpl
        ) if self.sourcetype else default_tmpl
        default_varnames = getattr(self, '{}_varnames'.format(key), {})
        varnames = self.config.get(
            '{}_{}_varnames'.format(self.sourcetype, key),
            default_varnames
        ) if self.sourcetype else default_varnames
        return tmpl.format(**dict((k, getattr(self, k)) for k in varnames))
    
    @cached_property
    def entries(self):
        return sorted(
            self._get_entries(),
            key=attrgetter(self.entry_sort_key),
            reverse=self.entry_sort_reversed
        )
    
    def _get_entries(self):
        raise NotImplementedError
    
    @cached_property
    def urlpath(self):
        return self._get_urlpath()
    
    def _get_urlpath(self):
        if self.urlshort is not None:
            return "{}index".format(self.urlshort)
        raise NotImplementedError
    
    @cached_property
    def link_attrs(self):
        return self._get_link_attrs()
    
    def _get_link_attrs(self):
        return dict(
            urlshort=self.urlshort,
            urlpath=self.urlpath,
            title=self.title,
            heading=self.heading,
            count=len(self.entries)
        )
    
    @cached_property
    def sourcetype_sources(self):
        return self._get_sourcetype_sources()
    
    def _get_sourcetype_sources(self):
        return []
    
    @cached_property
    def sourcetype_index(self):
        if self.sourcetype_sources:
            return self.sourcetype_sources.index(self)
        return -1
    
    def _get_next_source(self):
        i = self.sourcetype_index
        if (i > -1) and (i < (len(self.sourcetype_sources) - 1)):
            return self.sourcetype_sources[i + 1]
        return None
    
    def _get_prev_source(self):
        i = self.sourcetype_index
        if i > 0:
            return self.sourcetype_sources[i - 1]
        return None


class BlogIndex(BlogEntries):
    
    sourcetype = 'blog'
    urlshort = "/"
    default_title = "Home"
    default_heading = "Home Page"
    
    def _get_entries(self):
        return self.blog.all_entries


# PAGES


def prefixed_keys(mapping, prefix):
    """Return dict with all keys prefixed by ``prefix``.
    """
    tmpl = prefix + "{}"
    return mapping.__class__(
        (tmpl.format(key), mapping[key])
        for key in mapping
    )


class BlogEntryParams(object):
    """Convenience object to store format parameters.
    """
    
    def __init__(self, mapping=None, **kwargs):
        self.update(mapping, **kwargs)
    
    def update(self, mapping=None, **kwargs):
        for obj in (mapping, kwargs):
            if obj:
                for k, v in obj.items():
                    setattr(self, k, v)
    
    def get(self, key, default=None):
        return getattr(self, key, default)


@extendable
class BlogPage(BlogObject):
    """Single page containing one or more entries.
    """
    
    config_vars = dict(
        no_entries=('no_entries_content', "<p>No entries found!</p>"),
        source_link_template='<a href="{urlshort}">{title}</a>',
        source_link_sep="&nbsp;&nbsp;"
    )
    
    def __init__(self, blog, source, format):
        BlogObject.__init__(self, blog)
        self.source = source
        self.title = self.source.title
        self.heading = self.source.heading
        if isinstance(source, BlogEntry):
            self.entries = [source]
        else:
            self.entries = source.entries
        self.format = format
        self.urlpath = "{0}.{1}".format(source.urlpath, format)
    
    @cached_property
    def filepath(self):
        return os.path.join(
            *self.urlpath[1:].split('/')
        )
    
    # The format_entries generator is not extendable;
    # mixins can override _get_format_entries
    
    @extendable_method()
    def entry_params(self, entry):
        return BlogEntryParams()
    
    @memoize_generator
    def format_entries(self):
        return self._get_format_entries()
    
    def _get_format_entries(self):
        for entry in self.entries:
            yield entry.formatted(self.format, self.entry_params(entry))
    
    @extendable_property()
    def body(self):
        if self.entries:
            return newline.join(self.format_entries())
        return self.no_entries
    
    @extendable_property()
    def template(self):
        return self.template_data("page", self.format)
    
    @extendable_property()
    def link_source(self):
        return self.source
    
    @cached_property
    def source_links(self):
        """Return HTML links to next and previous sources.
        """
        source = self.link_source
        return tuple(
            self.source_link_template.format(**s.link_attrs) if s else ""
            for s in ((source.next_source, source.prev_source) if source else (None, None))
        )
    
    @extendable_property()
    def attrs(self):
        links = (link_next_source, link_prev_source) = self.source_links
        metadata = dict(
            title=self.title,
            heading=self.heading,
            entries=self.body,
            sourcelinks=self.source_link_sep.join(link for link in links if link),
            sourcelink_next=link_next_source,
            sourcelink_prev=link_prev_source
        )
        return dict(
            prefixed_keys(self.blog.metadata, 'blog_'),
            sys_gen_name='simpleblog3',
            sys_gen_uri="http://pypi.python.org/pypi/simpleblog3",
            sys_gen_version=__version__,
            **prefixed_keys(metadata, 'page_')
        )
    
    @extendable_property()
    def formatted(self):
        return self.template.format(**self.attrs)
    
    @cached_property
    def encoded(self):
        return encode(self.formatted, self.blog.metadata['charset'])


# BLOG


class BlogMetadataError(BlogError):
    pass


@extendable
class Blog(BlogObject):
    """The entire blog.
    """
    
    config_vars = dict(
        index_formats=dict(
            vartype=set,
            default=["html"]),
        entry_formats=dict(
            vartype=set,
            default=["html"])
    )
    
    def __init__(self, config, filename=None):
        self.blog = self
        self.config = config
        self.metadata = {}
        load_blogfile(filename, "blog", self.metadata)
        for key in self.required_metadata:
            if key not in self.metadata:
                raise BlogMetadataError("{} missing from blog metadata".format(key))
        for key, value in self.default_metadata.items():
            self.metadata.setdefault(key, value.format(**self.metadata))
    
    @extendable_property()
    def required_metadata(self):
        return set()
    
    @extendable_property()
    def default_metadata(self):
        return dict(
            charset='utf-8'
        )
    
    @cached_method
    def filter_entries(self, path):
        return suffixed_items(os.listdir(path), self.entry_ext)
    
    @cached_property
    def entry_class(self):
        return extension_types['entry']
    
    @extendable_property()
    def all_entries(self):
        return [
            self.entry_class(self, name)
            for name in self.filter_entries(self.entries_dir)
        ]
    
    @extendable_method()
    def index_entries(self, format):
        return BlogIndex(self)
    
    @extendable_property()
    def sources(self):
        
        return [
            (self.index_entries(format), format)
            for format in self.index_formats
        ] + [
            (entry, format)
            for entry in self.all_entries
            for format in self.entry_formats
        ]
    
    @cached_property
    def page_class(self):
        return extension_types['page']
    
    @extendable_property()
    def pages(self):
        return [
            self.page_class(self, source, format)
            for source, format in self.sources
        ]


# INITIALIZATION


def initialize(config):
    extensions = config.get("extensions", ())
    if extensions:
        from simpleblog.ext import load
        load(config, extensions)


def load_blog(opts):
    config = BlogConfig(opts.configfile)
    initialize(config)
    blog = extension_types['blog'](config, opts.blogfile)
    return config, blog



#!/usr/bin/env python3
"""
Module COPYRIGHT -- Simple Blog Copyright Extension
Sub-Package SIMPLEBLOG.EXTENSIONS
Copyright (C) 2012-2013 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information
"""

from plib.stdlib.decotools import cached_method

from simpleblog.extensions import BlogExtension


class CopyrightExtension(BlogExtension):
    """Add copyright information to blog metadata.
    """
    
    config_vars = dict(
        copyright_template="Copyright {start_year}-{end_year}",
        copyright_display_template="Copyright &copy; {start_year}-{end_year}",
        copyright_start_year=None,
        copyright_end_year=None
    )
    
    @cached_method
    def entry_years(self, blog):
        return [entry.timestamp.year for entry in blog.all_entries]
    
    @cached_method
    def start_year(self, blog):
        return min(self.entry_years(blog))
    
    @cached_method
    def end_year(self, blog):
        return max(self.entry_years(blog))
    
    def blog_mod_default_metadata(self, blog, data):
        params = dict(
            data,
            start_year=self.copyright_start_year or self.start_year(blog),
            end_year=self.copyright_end_year or self.end_year(blog)
        )
        data.update(
            copyright=self.copyright_template.format(**params),
            copyright_display=self.copyright_display_template.format(**params),
        )
        return data



#!/usr/bin/env python3 -u
"""
Setup script for SIMPLEBLOG package
Copyright (C) 2012-2013 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information
"""

from simpleblog import __version__ as version

name = "simpleblog3"
description = "A simple Python 3 blogging system."

startline = 5
endspec = "Copyright and License"

author = "Peter A. Donis"
author_email = "peterdonis@alum.mit.edu"

data_dirs = ["examples"]

dev_status = "Beta"

license = "GPLv2"

classifiers="""
Environment :: Console
Intended Audience :: Developers
Intended Audience :: End Users/Desktop
Operating System :: POSIX
Operating System :: POSIX :: Linux
"""

requires = """
plib3.stdlib (>=0.9.1)
"""

rst_header_template = """**{basename}** for {name} {version}

:Author:        {author}
:Release Date:  {releasedate}
"""


if __name__ == '__main__':
    from distutils.core import setup
    from setuputils import convert_md_to_rst, current_date, setup_vars
    
    convert_md_to_rst(rst_header_template,
        startline=2,
        name=name.upper(),
        version=version,
        author=author,
        releasedate=current_date("%d %b %Y")
    )
    
    setup(**setup_vars(globals()))



#!/usr/bin/env python3
"""
Module INDEXES -- Simple Blog Index Generator Extension
Sub-Package SIMPLEBLOG.EXTENSIONS
Copyright (C) 2012-2013 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information
"""

from operator import attrgetter
from re import compile, sub

from plib.stdlib.decotools import cached_property

from simpleblog import BlogObject, BlogPage, newline
from simpleblog.extensions import BlogExtension


rexp = compile(r"[^A-Za-z\ ]")


class BlogIndexPage(BlogPage):
    """Specialized page class for page with links instead of entries.
    """
    
    config_vars = dict(
        index_link_template="{link}",
        index_link_suffix_template="",
        link_index_title_template="{heading}",
        link_index_heading_template="{label} Index",
        link_index_heading_alpha="Alphabetical",
        link_index_heading_chrono="Chronological",
        link_index_heading_key="Key",
        link_index_sep="<br>"
    )
    
    def __init__(self, blog, format, alpha):
        BlogObject.__init__(self, blog)
        self.source = self.entries = None
        self.format = format
        self.alpha = alpha
        self.urlpath = "/index-{0}.{1}".format(
            "key" if alpha is None else "alpha" if alpha else "chrono",
            format
        )
        self.heading = self.link_index_heading_template.format(
            label=(
                self.link_index_heading_key if alpha is None
                else self.link_index_heading_alpha if alpha
                else self.link_index_heading_chrono
            )
        )
        self.title = self.link_index_title_template.format(
            heading=self.heading
        )
    
    @cached_property
    def indexlinks_template(self):
        return self.template_data("index", "links")
    
    @cached_property
    def no_entries(self):
        # Hack to populate the page with links instead of entries
        if self.alpha is None:
            key = label = attrgetter('cachekey')
            suffix = lambda entry: ""
            reverse = False
        else:
            label = attrgetter('title')
            if self.alpha:
                # Remove characters that aren't spaces or alphanumerics for sorting
                key = lambda entry: sub(rexp, "", entry.title)
            else:
                key = attrgetter('timestamp')
            suffix = lambda entry: self.index_link_suffix_template.format(entry)
            reverse = not self.alpha
        return self.indexlinks_template.format(
            index_links=("{}{}".format(self.link_index_sep, newline)).join([
                self.index_link_template.format(
                    link='<a href="{0}.{1}">{2}</a>{3}'.format(
                        entry.urlpath, self.format, label(entry), suffix(entry)
                    )
                ) for entry in sorted(self.blog.all_entries, key=key, reverse=reverse)
            ])
        )


class IndexesExtension(BlogExtension):
    """Add alphabetical, chronological, and/or key link index pages to blog.
    """
    
    config_vars = dict(
        link_index_alpha=False,
        link_index_chrono=True,
        link_index_key=False,
        link_index_formats=dict(
            vartype=set,
            default=["html"])
    )
    
    @cached_property
    def link_index_alphas(self):
        result = []
        if self.link_index_key:
            result.append(None)
        if self.link_index_chrono:
            result.append(False)
        if self.link_index_alpha:
            result.append(True)
        return result
    
    def blog_mod_pages(self, blog, pages):
        pages.extend(
            BlogIndexPage(blog, format, alpha)
            for format in self.link_index_formats
            for alpha in self.link_index_alphas
        )
        return pages



#!/usr/bin/env python3
"""
Module CACHING -- Simple Blog Caching Mechanism
Package SIMPLEBLOG
Copyright (C) 2012-2013 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information
"""

import os
import codecs
from functools import wraps

from plib.stdlib.decotools import cached_property

from simpleblog import BlogObject


class BlogCache(BlogObject):
    """Cache for data associated with blog entries.
    
    This allows the blog structure and metadata to be assembled
    without having to stat or open and load any entry files.
    """
    
    def __init__(self, blog, cachename, reverse=False, objtype=None, sep=' ', encoding='utf-8'):
        BlogObject.__init__(self, blog)
        self.cachename = cachename
        self.reverse = reverse
        self.objtype = objtype
        self.sep = sep
        self.encoding = encoding
    
    @cached_property
    def cache_dir(self):
        return self.config.get('cache_dir', self.entries_dir)
    
    @cached_property
    def filename(self):
        return os.path.join(self.cache_dir, self.cachename)
    
    @cached_property
    def cache(self):
        try:
            with codecs.open(self.filename, 'r', self.encoding) as f:
                lines = f.readlines()
        except IOError:
            return {}
        else:
            if self.reverse:
                r = lambda line: tuple(reversed(line.strip().rsplit(self.sep, 1)))
            else:
                r = lambda line: tuple(line.strip().split(self.sep, 1))
            if self.objtype:
                e = lambda t: (t[0], self.objtype(t[1]))
                expr = (e(r(line)) for line in lines)
            else:
                expr = (r(line) for line in lines)
            return dict(expr)
    
    def save(self):
        items = iter(self.cache.items())
        if self.reverse:
            items = ((v, k) for k, v in items)
        lines = sorted("{0}{1}{2}\n".format(str(a), self.sep, str(b)) for a, b in items)
        with codecs.open(self.filename, 'w', self.encoding) as f:
            f.writelines(lines)


cache_map = {}


def cached(cachename, reverse=False, objtype=None, sep=' '):
    """Decorator for blog entry properties that should be cached.
    """
    
    def decorator(f):
        @wraps(f)
        def fcache(self, *args, **kwargs):
            try:
                cacheobj = cache_map[cachename]
            except KeyError:
                cacheobj = cache_map[cachename] = BlogCache(
                    self.blog,
                    cachename,
                    reverse,
                    objtype,
                    sep,
                    self.blog.metadata['charset']
                )
            cache = cacheobj.cache
            try:
                return cache[self.cachekey]
            except KeyError:
                value = f(self, *args, **kwargs)
                if cacheobj.objtype is not None:
                    value = cacheobj.objtype(value)
                cache[self.cachekey] = value
                cacheobj.save()
                return value
        return fcache
    return decorator
